@@ -1,6 +1,6 @@
 # DIF MANTEL-HAENZEL
 
-difMH<-function (Data, group, focal.name, MHstat = "MHChisq", correct = TRUE, 
+difMH<-function (Data, group, focal.name, MHstat = "MHChisq", correct = TRUE, exact=FALSE,
     alpha = 0.05, purify = FALSE, nrIter = 10, save.output = FALSE, 
     output = c("out", "default")) 
 {
@@ -27,10 +27,89 @@ difMH<-function (Data, group, focal.name, MHstat = "MHChisq", correct = TRUE,
         Group[gr == focal.name] <- 1
         Q <- switch(MHstat, MHChisq = qchisq(1 - alpha, 1), logOR = qnorm(1 - 
             alpha/2))
-        if (is.null(Q) == TRUE) 
-            stop("'MHstat' argument not valid", call. = FALSE)
-        if (purify == FALSE) {
-            PROV <- mantelHaenszel(DATA, Group, correct = correct)
+        if (is.null(Q)) stop("'MHstat' argument not valid", call. = FALSE)
+
+if (exact){
+        if (!purify) {
+            PROV <- mantelHaenszel(DATA, Group, correct = correct,exact=exact)
+                STATS <- PROV$resMH
+            if (min(PROV$Pval) >=alpha) DIFitems <- "No DIF item detected"
+            else DIFitems <- (1:ncol(DATA))[PROV$Pval < alpha]
+            RES <- list(MH = STATS, Pval=PROV$Pval, alpha = alpha, DIFitems = DIFitems, 
+                correct = correct, exact=exact, purification = purify, names = colnames(DATA), 
+                save.output = save.output, output = output)
+        }
+        else {
+            nrPur <- 0
+            difPur <- NULL
+            noLoop <- FALSE
+            prov1 <- mantelHaenszel(DATA, Group, correct = correct,exact=exact)
+            stats1 <- prov1$resMH
+            if (min(prov1$Pval)>=alpha) {
+                DIFitems <- "No DIF item detected"
+                noLoop <- TRUE
+            }
+            else {
+                dif <- (1:ncol(DATA))[prov1$Pval<alpha]
+                difPur <- rep(0, length(stats1))
+                difPur[dif] <- 1
+                repeat {
+                  if (nrPur >= nrIter) 
+                    break
+                  else {
+                    nrPur <- nrPur + 1
+                    nodif <- NULL
+                    if (is.null(dif)) 
+                      nodif <- 1:ncol(DATA)
+                    else {
+                      for (i in 1:ncol(DATA)) {
+                        if (sum(i == dif) == 0) 
+                          nodif <- c(nodif, i)
+                      }
+                    }
+                    prov2 <- mantelHaenszel(DATA, Group, correct = correct, 
+                      anchor = nodif,exact=exact)
+                    stats2 <- prov2$resMH
+                    if (min(prov2$Pval)>=alpha) dif2 <- NULL
+                    else dif2 <- (1:ncol(DATA))[prov2$Pval<alpha]
+                    difPur <- rbind(difPur, rep(0, ncol(DATA)))
+                    difPur[nrPur + 1, dif2] <- 1
+                    if (length(dif) != length(dif2)) 
+                      dif <- dif2
+                    else {
+                      dif <- sort(dif)
+                      dif2 <- sort(dif2)
+                      if (sum(dif == dif2) == length(dif)) {
+                        noLoop <- TRUE
+                        break
+                      }
+                      else dif <- dif2
+                    }
+                  }
+                }
+                stats1 <- stats2
+                prov1 <- prov2
+                DIFitems <- (1:ncol(DATA))[prov1$Pval<alpha]
+            }
+            if (!is.null(difPur)) {
+                ro <- co <- NULL
+                for (ir in 1:nrow(difPur)) ro[ir] <- paste("Step", 
+                  ir - 1, sep = "")
+                for (ic in 1:ncol(difPur)) co[ic] <- paste("Item", 
+                  ic, sep = "")
+                rownames(difPur) <- ro
+                colnames(difPur) <- co
+            }
+            RES <- list(MH = stats1, Pval=prov1$Pval, alpha = alpha, DIFitems = DIFitems, 
+                correct = correct, exact=exact, purification = purify, nrPur = nrPur, 
+                difPur = difPur, convergence = noLoop, names = colnames(DATA), 
+                save.output = save.output, output = output)
+        }
+}
+else{
+
+        if (!purify) {
+            PROV <- mantelHaenszel(DATA, Group, correct = correct,exact=exact)
             if (MHstat == "MHChisq") 
                 STATS <- PROV$resMH
             else STATS <- log(PROV$resAlpha)/sqrt(PROV$varLambda)
@@ -40,14 +119,14 @@ difMH<-function (Data, group, focal.name, MHstat = "MHChisq", correct = TRUE,
             RES <- list(MH = STATS, alphaMH = PROV$resAlpha, 
                 varLambda = PROV$varLambda, MHstat = MHstat, 
                 alpha = alpha, thr = Q, DIFitems = DIFitems, 
-                correct = correct, purification = purify, names = colnames(DATA), 
+                correct = correct, exact=exact, purification = purify, names = colnames(DATA), 
                 save.output = save.output, output = output)
         }
         else {
             nrPur <- 0
             difPur <- NULL
             noLoop <- FALSE
-            prov1 <- mantelHaenszel(DATA, Group, correct = correct)
+            prov1 <- mantelHaenszel(DATA, Group, correct = correct,exact=exact)
             if (MHstat == "MHChisq") 
                 stats1 <- prov1$resMH
             else stats1 <- log(prov1$resAlpha)/sqrt(prov1$varLambda)
@@ -74,7 +153,7 @@ difMH<-function (Data, group, focal.name, MHstat = "MHChisq", correct = TRUE,
                       }
                     }
                     prov2 <- mantelHaenszel(DATA, Group, correct = correct, 
-                      anchor = nodif)
+                      anchor = nodif,exact=exact)
                     if (MHstat == "MHChisq") 
                       stats2 <- prov2$resMH
                     else stats2 <- log(prov2$resAlpha)/sqrt(prov2$varLambda)
@@ -113,10 +192,11 @@ difMH<-function (Data, group, focal.name, MHstat = "MHChisq", correct = TRUE,
             RES <- list(MH = stats1, alphaMH = prov1$resAlpha, 
                 varLambda = prov1$varLambda, MHstat = MHstat, 
                 alpha = alpha, thr = qchisq(1 - alpha, 1), DIFitems = DIFitems, 
-                correct = correct, purification = purify, nrPur = nrPur, 
+                correct = correct, exact=exact,purification = purify, nrPur = nrPur, 
                 difPur = difPur, convergence = noLoop, names = colnames(DATA), 
                 save.output = save.output, output = output)
         }
+}
         class(RES) <- "MH"
         return(RES)
     }
@@ -133,10 +213,10 @@ difMH<-function (Data, group, focal.name, MHstat = "MHChisq", correct = TRUE,
 
 
 
-
 # METHODS
 plot.MH<-function (x, pch = 8, number = TRUE, col = "red", save.plot=FALSE,save.options=c("plot","default","pdf"),...) 
 {
+if (x$exact) stop("Error: plot is not available with exact Mantel-Haenszel test",call.=FALSE)
 internalMH<-function(){
     res <- x
     if (res$MHstat == "MHChisq") 
@@ -183,7 +263,7 @@ dev.off()
 }
 if (plotype==2){
 {
-jpeg(file=fileName)
+jpeg(filename=fileName)
 internalMH()
 }
 dev.off()
@@ -194,6 +274,8 @@ cat("The plot was captured and saved into","\n"," '",fileName,"'","\n","\n",sep=
 else cat("The plot was not captured!","\n",sep="")
 }
 
+
+
 print.MH<-function (x, ...) 
 {
     res <- x
@@ -208,16 +290,16 @@ print.MH<-function (x, ...)
     else pur <- "without "
     cat(corr, "continuity correction and ", pur, "item purification", 
         "\n", "\n", sep = "")
-    if (res$purification == TRUE) {
+    if (res$exact) cat("Results based on exact inference","\n","\n")
+    else cat("Results based on asymptotic inference","\n","\n")
+    if (res$purification) {
         if (res$nrPur <= 1) 
             word <- " iteration"
         else word <- " iterations"
-        if (res$convergence == FALSE) {
-            cat("WARNING: no item purification convergence after ", 
-                res$nrPur, word, "\n", sep = "")
+        if (!res$convergence) {
+            cat("WARNING: no item purification convergence after ",res$nrPur, word, "\n", sep = "")
             loop <- NULL
-            for (i in 1:res$nrPur) loop[i] <- sum(res$difPur[1, 
-                ] == res$difPur[i + 1, ])
+            for (i in 1:res$nrPur) loop[i] <- sum(res$difPur[1,] == res$difPur[i + 1, ])
             if (max(loop) != length(res$MH)) 
                 cat("(Note: no loop detected in less than ", 
                   res$nrPur, word, ")", "\n", sep = "")
@@ -230,18 +312,21 @@ print.MH<-function (x, ...)
         else cat("Convergence reached after ", res$nrPur, word, 
             "\n", "\n", sep = "")
     }
-    met <- switch(res$MHstat, MHChisq = "Mantel-Haenszel Chi-square statistic:", 
+if (res$exact) met<-"Exact statistic:"
+else    met <- switch(res$MHstat, MHChisq = "Mantel-Haenszel Chi-square statistic:", 
         logOR = "Log odds-ratio statistic:")
     cat(met, "\n", "\n")
-    if (res$MHstat == "MHChisq") 
-        pval <- round(1 - pchisq(res$MH, 1), 4)
+if (res$exact)  pval<-round(res$Pval,4)
+else{
+    if (res$MHstat == "MHChisq") pval <- round(1 - pchisq(res$MH, 1), 4)
     else pval <- round(2 * (1 - pnorm(abs(res$MH))), 4)
+}
     symb <- symnum(pval, c(0, 0.001, 0.01, 0.05, 0.1, 1), symbols = c("***", 
         "**", "*", ".", ""))
-    m1 <- cbind(round(res$MH, 4), pval)
+    if (!res$exact) m1 <- cbind(round(res$MH, 4), pval)
+else m1 <- cbind(round(res$MH), pval)
     m1 <- noquote(cbind(format(m1, justify = "right"), symb))
-    if (is.null(res$names) == FALSE) 
-        rownames(m1) <- res$names
+    if (!is.null(res$names)) rownames(m1) <- res$names
     else {
         rn <- NULL
         for (i in 1:nrow(m1)) rn[i] <- paste("Item", i, sep = "")
@@ -252,11 +337,10 @@ print.MH<-function (x, ...)
     cat("\n")
     cat("Signif. codes: 0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1 ", 
         "\n")
-    cat("\n", "Detection threshold: ", round(res$thr, 4), " (significance level: ", 
+if (res$exact) cat("\n","Significance level: ",res$alpha, "\n", "\n", sep = "")
+else cat("\n", "Detection threshold: ", round(res$thr, 4), " (significance level: ", 
         res$alpha, ")", "\n", "\n", sep = "")
-    if (is.character(res$DIFitems) == TRUE) 
-        cat("Items detected as DIF items:", res$DIFitems, "\n", 
-            "\n")
+    if (is.character(res$DIFitems)) cat("Items detected as DIF items:", res$DIFitems, "\n", "\n")
     else {
         cat("Items detected as DIF items:", "\n")
         m2 <- cbind(rownames(m1)[res$DIFitems])
@@ -265,6 +349,7 @@ print.MH<-function (x, ...)
         print(m2, quote = FALSE)
         cat("\n", "\n")
     }
+if (!res$exact){
     cat("Effect size (ETS Delta scale):", "\n", "\n")
     cat("Effect size code:", "\n")
     cat(" 'A': negligible effect", "\n")
@@ -288,7 +373,8 @@ print.MH<-function (x, ...)
     cat("\n")
     cat("Effect size codes: 0 'A' 1.0 'B' 1.5 'C'", "\n")
     cat(" (for absolute values of 'deltaMH')", "\n", "\n")
-    if (x$save.output==FALSE) cat("Output was not captured!","\n")
+}
+    if (!x$save.output) cat("Output was not captured!","\n")
     else {
 if (x$output[2]=="default") wd<-paste(getwd(),"/",sep="")
 else wd<-x$output[2]
